package pro.skywalking.utils;

import java.util.function.IntConsumer;
import java.util.stream.IntStream;
import pro.skywalking.configuration.auth.EncryptUtils;

public class SnowflakeIdWorker {

   private final long twepoch = 1420041600000L;
   private final long workerIdBits = 5L;
   private final long datacenterIdBits = 5L;
   private final long maxWorkerId = 31L;
   private final long maxDatacenterId = 31L;
   private final long sequenceBits = 12L;
   private final long workerIdShift = 12L;
   private final long datacenterIdShift = 17L;
   private final long timestampLeftShift = 22L;
   private final long sequenceMask = 4095L;
   private long workerId;
   private long datacenterId;
   private long sequence = 0L;
   private long lastTimestamp = -1L;
   private static SnowflakeIdWorker snowflakeIdWorker = new SnowflakeIdWorker(0L, 6L);


   public SnowflakeIdWorker(long workerId, long datacenterId) {
      if(workerId <= 31L && workerId >= 0L) {
         if(datacenterId <= 31L && datacenterId >= 0L) {
            this.workerId = workerId;
            this.datacenterId = datacenterId;
         } else {
            throw new IllegalArgumentException(String.format("datacenter Id can\'t be greater than %d or less than 0", new Object[]{Long.valueOf(31L)}));
         }
      } else {
         throw new IllegalArgumentException(String.format("worker Id can\'t be greater than %d or less than 0", new Object[]{Long.valueOf(31L)}));
      }
   }

   public synchronized long nextId() {
      long timestamp = this.timeGen();
      if(timestamp < this.lastTimestamp) {
         throw new RuntimeException(String.format("Clock moved backwards.  Refusing to generate id for %d milliseconds", new Object[]{Long.valueOf(this.lastTimestamp - timestamp)}));
      } else {
         if(this.lastTimestamp == timestamp) {
            this.sequence = this.sequence + 1L & 4095L;
            if(this.sequence == 0L) {
               timestamp = this.tilNextMillis(this.lastTimestamp);
            }
         } else {
            this.sequence = 0L;
         }

         this.lastTimestamp = timestamp;
         return timestamp - 1420041600000L << 22 | this.datacenterId << 17 | this.workerId << 12 | this.sequence;
      }
   }

   protected long tilNextMillis(long lastTimestamp) {
      long timestamp;
      for(timestamp = this.timeGen(); timestamp <= lastTimestamp; timestamp = this.timeGen()) {
         ;
      }

      return timestamp;
   }

   protected long timeGen() {
      return System.currentTimeMillis();
   }

   public static Long uniqueSequence() {
      return Long.valueOf(snowflakeIdWorker.nextId());
   }

   public static String uniqueMainId() {
      return String.valueOf(snowflakeIdWorker.nextId());
   }

   public static String unique32MainId() {
      return EncryptUtils.md5String(String.valueOf(snowflakeIdWorker.nextId()));
   }

   public static String uniqueStringSequence() {
      return String.valueOf(snowflakeIdWorker.nextId());
   }

   public static void main(String[] args) {
      Integer range = Integer.valueOf(4);
      IntStream.rangeClosed(1, range.intValue()).forEach((mainId) -> {
         String mainId = uniqueMainId();
         System.out.println(mainId);
         System.out.println("长度:" + mainId.length());
      });
   }

}
